@@ -2,6 +2,11 @@
 // sys/bit.h: Bit Vector Library, system level
 // ============================================================================
 //
+// 11/28/01 <gerstl>	Removed constructors from shadow _longlong_
+// 11/17/01 <gerstl>	Added include/define for vptr from GL_System.h
+// 09/27/01 <gerstl>	Added constructor from unqualified char
+// 08/10/01 <gerstl>	Introduced len_t, bnd_t; changed shift cnt to unsigned
+//			Fixed bug with slicing of a bit slice (_BITSLICE macro)
 // 06/01/01 <gerstl>	Resolved conflict of method names with ANSI (keywords?)
 // 05/24/01 <gerstl>	Initial version
 //
@@ -16,6 +21,10 @@
 #include <longlong.h>
 
 
+/* Where is the virtual method table pointer? */
+#define GL_SYSTEM_VPTR_IS_LAST_IN_CLASS
+
+
 /* Enable assertions only if -DDEBUG (but executed only if not -DNDEBUG ...) */
 #ifdef DEBUG
 # define _BITASSERT
@@ -34,18 +43,21 @@
 /* --- Some macros --- */
 
 // Length of a bit vector, given boundaries
-#define _BITLEN(l, r)           ((l) > (r) ? ((l)-(r)+1) : ((r)-(l)+1))
+#define _BITLEN(l, r)	((_bit::len_t)((l)>(r) ? ((l)-(r)+1) : ((r)-(l)+1)))
 
+// Mapping of arbitrary bit index to index into zero-aligned vector
+#define _BITMAP(l,r,i)	((_bit::len_t)((l)>(r)? ((i) - (r)) : ((r) - (i))))
+
+// Mapping of a zero-based index into a bit slice (reverse of _BITMAP)
+#define _BITSLICE(l,r,i)	((l)>(r)? (r) + (i) : (r) - (i))
+  
 // Mapping of bit index to data chunks
-#define _BITCHUNK(i)            ((int)((i) / _BITCHUNK_SIZE))
+#define _BITCHUNK(i)		((int)((i) / _BITCHUNK_SIZE))
 
 // Min / Max operations
 #define _MAXLEN(a,b)	        ((a) > (b) ? (a) : (b))
 #define _MINLEN(a,b)	        ((a) < (b) ? (a) : (b))
 
-//// Mapping of arbitrary bit index to index into zero-aligned vector
-#define _BITMAP(l,r,i)          ((l) > (r)? ((i) - (r)) : ((r) - (i)))
-
 // sign[] function
 #define _BITSIGN(l,r)           ((l) == (r)? 0 : ((l) > (r)? 1 : -1))
 
@@ -56,7 +68,7 @@
 #define _SIZEOFBIT(len)         (_BITCHUNKS(len)*sizeof(_bit::chunk))
 
 // Size of bit chunk
-#define _BITCHUNK_SIZE          (int)((sizeof(_bit::chunk)*CHAR_BIT))
+#define _BITCHUNK_SIZE          ((int)((sizeof(_bit::chunk)*CHAR_BIT)))
 
 // Size of the complete bit vector class (including virtual table pointer, 
 // same as sizeof(bit<len>) but at runtime)
@@ -138,15 +150,19 @@ class _bitslice;
 class _bit
 {
 public:
-  // Type definition of bit vector chunks
+  // Type definition of bit vector chunk
   union chunk {
     signed int   sval;
     unsigned int uval;
   };
+  
+  // Type definition for bit vector length, bounds
+  typedef unsigned int len_t;
+  typedef signed   int bnd_t;
 
 protected:
   chunk *vec;
-  const int  len;
+  const len_t len;
   const bool usgn;
   
 public:
@@ -155,61 +171,64 @@ public:
                { for (int i = 0; i < _BITCHUNKS(len); i++) vec[i] = c.vec[i]; }
 
   // Constructor for an empty bit vector
-  _bit(int length, bool usign): len(length), usgn(usign), isDynamic(init()) { }
+  _bit(len_t length, bool usign)
+    : len(length), usgn(usign), isDynamic(init())                           { }
 
   // Constructor for a reference to an existing vector
-  _bit(chunk* ref, int length, bool usign)
+  _bit(chunk* ref, len_t length, bool usign)
     : vec(ref), len(length), usgn(usign), isDynamic(false)                  { }
      
   // Construct from a bit slice
-  _bit(const _bitslice& s, int length, bool usign)
+  _bit(const _bitslice& s, len_t length, bool usign)
     : len(length), usgn(usign), isDynamic(init())                   { set(s); }
   
   // Construct an operation result
   inline _bit(const _bit& op1, char type);                    // Unary
-  inline _bit(const _bit& op1, char type, int sh);            // Shift
+  inline _bit(const _bit& op1, char type, unsigned int sh);   // Shift
   inline _bit(const _bit& op1, char type, const _bit& op2);   // Binary
   inline _bit(const _bit& op1, const _bit& op2);              // Concatenation
 
 #ifdef _BIT_TEMPLATE_OPERATORS
   // Operation results as a reference
-  inline _bit(chunk* ref, int length, bool usign,
+  inline _bit(chunk* ref, len_t length, bool usign,
 	      const _bit& op1, char type);                    // Unary
-  inline _bit(chunk* ref, int length, bool usign,
-	      const _bit& op1, char type, int sh);            // Shift
-  inline _bit(chunk* ref, int length, bool usign, 
+  inline _bit(chunk* ref, len_t length, bool usign,
+	      const _bit& op1, char type, unsigned int sh);   // Shift
+  inline _bit(chunk* ref, len_t length, bool usign, 
 	      const _bit& op1, char type, const _bit& op2);   // Binary
-  inline _bit(chunk* ref, int length, bool usign,
+  inline _bit(chunk* ref, len_t length, bool usign,
 	      const _bit& op1, const _bit& op2);              // Concatenation
 #endif  
   
   // Construct from bit string
-  inline _bit(const char* str, int length = 0, bool usign = false,
+  inline _bit(const char* str, len_t length = 0, bool usign = false,
 	      unsigned int base = 2);
 
   // Integer Initializers, length=0 picks native length
-  inline _bit(bool b,               int length=0, bool usign=_BITUSGN_BOOL);
-  inline _bit(signed char i,        int length=0, bool usign=_BITUSGN_CHAR);
-  inline _bit(unsigned char i,      int length=0, bool usign=_BITUSGN_UCHAR);
-  inline _bit(short int i,          int length=0, bool usign=_BITUSGN_SHORT);
-  inline _bit(unsigned short int i, int length=0, bool usign=_BITUSGN_USHORT);
-  inline _bit(int i,                int length=0, bool usign=_BITUSGN_INT);
-  inline _bit(unsigned int i,       int length=0, bool usign=_BITUSGN_UINT);
-  inline _bit(long i,               int length=0, bool usign=_BITUSGN_LONG);
-  inline _bit(unsigned long i,      int length=0, bool usign=_BITUSGN_ULONG);
-  inline _bit(long long i,          int length=0, bool usign=_BITUSGN_LLONG);
-  inline _bit(unsigned long long i, int length=0, bool usign=_BITUSGN_ULLONG);
-  inline _bit(double d,             int length,   bool usign=_BITUSGN_DBL);
-  inline _bit(long double d,        int length,   bool usign=_BITUSGN_LDBL);
+  inline _bit(bool b,               len_t length=0, bool usign=_BITUSGN_BOOL);
+  inline _bit(char i,               len_t length=0, bool usign=_BITUSGN_CHAR);
+    /* unqualified char is a "signed char" in SpecC (and vice versa) */
+  inline _bit(signed char i,        len_t length=0, bool usign=_BITUSGN_CHAR);
+  inline _bit(unsigned char i,      len_t length=0, bool usign=_BITUSGN_UCHAR);
+  inline _bit(short int i,          len_t length=0, bool usign=_BITUSGN_SHORT);
+  inline _bit(unsigned short i,     len_t length=0, bool usign=_BITUSGN_USHORT);
+  inline _bit(int i,                len_t length=0, bool usign=_BITUSGN_INT);
+  inline _bit(unsigned int i,       len_t length=0, bool usign=_BITUSGN_UINT);
+  inline _bit(long i,               len_t length=0, bool usign=_BITUSGN_LONG);
+  inline _bit(unsigned long i,      len_t length=0, bool usign=_BITUSGN_ULONG);
+  inline _bit(long long i,          len_t length=0, bool usign=_BITUSGN_LLONG);
+  inline _bit(unsigned long long i, len_t length=0, bool usign=_BITUSGN_ULLONG);
+  inline _bit(double d,             len_t length,   bool usign=_BITUSGN_DBL);
+  inline _bit(long double d,        len_t length,   bool usign=_BITUSGN_LDBL);
   
   // Destructor: delete if dynamic
   ~_bit()                                       { if(isDynamic) delete[] vec; }
 
   // Slicing
-  inline _bitslice operator[](int i) const;                 // [i]
-  inline _bitslice operator()(int sl, int sr) const;        // [left:right]
-  inline _bitslice operator()(int l, int r, int i) const;   // mapped versions
-  inline _bitslice operator()(int l, int r, int sl, int sr) const;
+  inline _bitslice operator[](len_t i) const;                   // [i]
+  inline _bitslice operator()(len_t sl, len_t sr) const;        // [left:right]
+  inline _bitslice operator()(bnd_t l, bnd_t r, bnd_t i) const; // mapped
+  inline _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const;
 
   // Assignments
   _bit& operator= (const _bit& op)                          { return set(op); }
@@ -240,8 +259,8 @@ public:
   _bit& operator%= (const _bit& op)          { return divOp(*this, op, this); }
 
   // Shift operators
-  _bit& operator<<= (int sh)                       { return shlOp(*this, sh); }
-  _bit& operator>>= (int sh)                       { return shrOp(*this, sh); }
+  _bit& operator<<= (unsigned int sh)              { return shlOp(*this, sh); }
+  _bit& operator>>= (unsigned int sh)              { return shrOp(*this, sh); }
 
   // Flip the bits in the vector (bv = ~bv)
   _bit& flip(void)                                     { return invOp(*this); }
@@ -261,7 +280,7 @@ public:
   inline static const _bit concat(const _bit& op1, const _bit& op2);
     
   // Copy slice of bits from some other vector
-  void copyBits(const _bit& src, int sl, int sr, int dl, int dr);
+  void copyBits(const _bit& src, len_t sl, len_t sr, len_t dl, len_t dr);
   
   // Explicit conversions
   inline int                toInt(void) const           { return vec[0].sval; }
@@ -276,7 +295,7 @@ public:
   // Get attributes
   chunk* addr(void) const                                      { return vec;  }
   bool   isUnsigned(void) const                                { return usgn; }
-  int    length(void) const                                    { return len;  }
+  len_t  length(void) const                                    { return len;  }
   
 protected:
   // Dynamically allocated storage?
@@ -304,7 +323,7 @@ protected:
   
   // Operation evaluation
   inline _bit& eval(const _bit& op1, char type);                  // Unary
-  inline _bit& eval(const _bit& op1, char type, int sh);          // Shift
+  inline _bit& eval(const _bit& op1, char type, unsigned int sh); // Shift
   inline _bit& eval(const _bit& op1, char type, const _bit& op2); // Binary
   inline _bit& eval(const _bit& op1, const _bit& op2);            // Concat
   
@@ -319,8 +338,8 @@ protected:
   inline _bit& subOp(const _bit& op1, const _bit& op2);
   inline _bit& subPo(const _bit& op1, const _bit& op2);
   inline _bit& concOp(const _bit& op1, const _bit& op2);
-         _bit& shlOp(const _bit& op, int sh, bool mask = false);
-         _bit& shrOp(const _bit& op, int sh);
+         _bit& shlOp(const _bit& op, unsigned int sh, bool mask = false);
+         _bit& shrOp(const _bit& op, unsigned int sh);
          _bit& mulOp(const _bit& op1, const _bit& op2);
          _bit& divOp(const _bit& n, const _bit& d, _bit* r = 0);
 
@@ -334,7 +353,7 @@ protected:
   inline _bit& fill(int chunk);
   
   // Fill bits with zeros or ones
-  void fill(int l, int r, int val);
+  void fill(len_t l, len_t r, int val);
 };
 
 
@@ -348,18 +367,21 @@ protected:
 class __bitslice
 {
 protected:
-  _bit ref;        // Reference to original vector
-  int  sl, sr;     // Indices of slice of original
+  typedef _bit::len_t len_t;
+  typedef _bit::bnd_t bnd_t;
+  
+  _bit  ref;        // Reference to original vector
+  len_t sl, sr;     // Indices of slice of original
   
 public:
   // Constructors
-  __bitslice(const _bit& o, int l, int r)
+  __bitslice(const _bit& o, len_t l, len_t r)
     : ref(o.addr(), o.length(), o.isUnsigned()), sl(l), sr(r)               { }
-  __bitslice(_bit::chunk* ovec, int olen, bool ousgn, int l, int r)
+  __bitslice(_bit::chunk* ovec, len_t olen, bool ousgn, len_t l, len_t r)
     : ref(ovec, olen, ousgn), sl(l), sr(r)                                  { }
             
   // Get attributes
-  int length(void) const  
+  len_t length(void) const  
               { return _BITLEN_SLICE(ref.length(), ref.isUnsigned(), sl, sr); }
 };
 
@@ -375,15 +397,15 @@ public:
   
 public:    
   // Bitbus element constructor
-  _bitbus_element(_bit& o, int l, int r, _bitbus_element* lst = 0)
+  _bitbus_element(_bit& o, len_t l, len_t r, _bitbus_element* lst = 0)
     : __bitslice(o, l, r), next(lst)                                        { }
-  _bitbus_element(_bit::chunk* ovec, int olen, bool ousgn, 
-		  int l, int r, _bitbus_element* lst = 0)
+  _bitbus_element(_bit::chunk* ovec, len_t olen, bool ousgn, 
+		  len_t l, len_t r, _bitbus_element* lst = 0)
     : __bitslice(ovec, olen, ousgn, l, r), next(lst)                        { }
   
   // Concatenation, slicing of bitbus list starting from this element
   _bitbus_element* concat(_bitbus_element* list);
-  _bitbus_element* slice(int l, int r);              // returns new list
+  _bitbus_element* slice(len_t l, len_t r);              // returns new list
     
   // Sync whole list starting with this element with the original vector
   void put(const _bit& buf);
@@ -404,10 +426,10 @@ protected:
   
 public:
   // Bit vector slice constructors
-  _bitslice(_bit& o, int l, int r, bool usign, _bitbus_element* bus = 0)
+  _bitslice(_bit& o, len_t l, len_t r, bool usign, _bitbus_element* bus = 0)
     : __bitslice(o, l, r), usgn(usign), lst(bus? bus->slice(l, r) : 0)      { }
-  _bitslice(_bit::chunk* ovec, int olen, bool ousgn, 
-	     int l, int r, bool usign, _bitbus_element* bus = 0)
+  _bitslice(_bit::chunk* ovec, len_t olen, bool ousgn, 
+	    len_t l, len_t r, bool usign, _bitbus_element* bus = 0)
     : __bitslice(ovec, olen, ousgn, l, r), usgn(usign),
       lst(bus? bus->slice(l, r) : 0)                                        { }
    
@@ -432,8 +454,8 @@ public:
   _bitslice& operator*=(const _bit& op)            { return put(res() *= op); }
   _bitslice& operator/=(const _bit& op)            { return put(res() /= op); }
   _bitslice& operator%=(const _bit& op)            { return put(res() %= op); }
-  _bitslice& operator<<= (int sh)                  { return put(res()<<= sh); }
-  _bitslice& operator>>= (int sh)                  { return put(res()>>= sh); }
+  _bitslice& operator<<= (unsigned int sh)         { return put(res()<<= sh); }
+  _bitslice& operator>>= (unsigned int sh)         { return put(res()>>= sh); }
 
   _bitslice& flip(void)                           { return put(res().flip()); }
   
@@ -444,14 +466,14 @@ public:
   _bit  operator-- (int i)           { _bit r(*this); put(--res()); return r; }
 
   // Slicing
-  inline _bitslice operator[](int i);
-  inline _bitslice operator()(int sl, int sr);
-  inline _bitslice operator()(int l, int r, int i);
-  inline _bitslice operator()(int l, int r, int sl, int sr);
-  inline const _bitslice operator[](int i) const;
-  inline const _bitslice operator()(int sl, int sr) const;
-  inline const _bitslice operator()(int l, int r, int i) const;
-  inline const _bitslice operator()(int l, int r, int sl, int sr) const;
+  inline _bitslice operator[](len_t i);
+  inline _bitslice operator()(len_t sl, len_t sr);
+  inline _bitslice operator()(bnd_t l, bnd_t r, bnd_t i);
+  inline _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr);
+  inline const _bitslice operator[](len_t i) const;
+  inline const _bitslice operator()(len_t sl, len_t sr) const;
+  inline const _bitslice operator()(bnd_t l, bnd_t r, bnd_t i) const;
+  inline const _bitslice operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const;
 
   // Zero test
   bool test(void) const                                { return res().test(); }
@@ -554,11 +576,11 @@ inline const _bit operator% (const _bit& op1, const _bit& op2) {
 
 /* --- Shifting --- */
 
-inline const _bit operator>> (const _bit& op1, int sh) {
+inline const _bit operator>> (const _bit& op1, unsigned int sh) {
   return _bit(op1, '>', sh);
 }
 
-inline const _bit operator<< (const _bit& op1, int sh) {
+inline const _bit operator<< (const _bit& op1, unsigned int sh) {
   return _bit(op1, '<', sh);
 }
 
@@ -654,76 +676,80 @@ inline bool operator>= (const _bit& op1, const _bit& op2) {
 
 /* --- Integer Initializers --- */
 
-inline _bit::_bit(bool b, int length, bool usign) 
+inline _bit::_bit(bool b, len_t length, bool usign) 
   : len(length? length : _BITLEN_BOOL), usgn(usign), isDynamic(init()) {
   set((int) b);
 }
 
-inline _bit::_bit(signed char i, int length, bool usign) 
+inline _bit::_bit(char i, len_t length, bool usign) 
+  : len(length? length : _BITLEN_CHAR), usgn(usign), isDynamic(init()) {
+  set((int)i);
+}
+
+inline _bit::_bit(signed char i, len_t length, bool usign) 
   : len(length? length : _BITLEN_CHAR), usgn(usign), isDynamic(init()) {
   set((signed int)i);
 }
 
-inline _bit::_bit(unsigned char i, int length, bool usign)
+inline _bit::_bit(unsigned char i, len_t length, bool usign)
   : len(length? length : _BITLEN_UCHAR), usgn(usign), isDynamic(init()) {
   set((unsigned int)i);
 }
 
-inline _bit::_bit(short int i, int length, bool usign) 
+inline _bit::_bit(short int i, len_t length, bool usign) 
   : len(length? length : _BITLEN_SHORT), usgn(usign), isDynamic(init()) {
   set((int) i);
 }
 
-inline _bit::_bit(unsigned short int i, int length, bool usign) 
+inline _bit::_bit(unsigned short int i, len_t length, bool usign) 
   : len(length? length : _BITLEN_USHORT), usgn(usign), isDynamic(init()) {
   set((unsigned int) i);
 }
 
-inline _bit::_bit(int i, int length, bool usign) 
+inline _bit::_bit(int i, len_t length, bool usign) 
   : len(length? length : _BITLEN_INT), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(unsigned int i, int length, bool usign) 
+inline _bit::_bit(unsigned int i, len_t length, bool usign) 
   : len(length? length : _BITLEN_UINT), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(long i, int length, bool usign) 
+inline _bit::_bit(long i, len_t length, bool usign) 
   : len(length? length : _BITLEN_LONG), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(unsigned long i, int length, bool usign) 
+inline _bit::_bit(unsigned long i, len_t length, bool usign) 
   : len(length? length : _BITLEN_ULONG), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(long long i, int length, bool usign) 
+inline _bit::_bit(long long i, len_t length, bool usign) 
   : len(length? length : _BITLEN_LLONG), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(unsigned long long i, int length, bool usign) 
+inline _bit::_bit(unsigned long long i, len_t length, bool usign) 
   : len(length? length : _BITLEN_ULLONG), usgn(usign), isDynamic(init()) {
   set(i);
 }
 
-inline _bit::_bit(double d, int length, bool usign)
+inline _bit::_bit(double d, len_t length, bool usign)
   : len(length), usgn(usign), isDynamic(init()) {
   set(d);
 }
 
-inline _bit::_bit(long double d, int length, bool usign) 
+inline _bit::_bit(long double d, len_t length, bool usign) 
   : len(length), usgn(usign), isDynamic(init()) {
   set(d);
 }
 
 
-
 /* --- String Initializier --- */
 
-inline _bit::_bit(const char* str, int length, bool usign, unsigned int base)
+inline _bit::_bit(const char* str, len_t length, bool usign, unsigned int base)
   : len(length? length : ((str[strlen(str)-1] == 'u')? (strlen(str)-1) : (strlen(str)))),
     usgn(usign || (length? false : (str[len] == 'u'))),
     isDynamic(init())
@@ -743,7 +769,7 @@ inline _bit::_bit(const _bit& op1, char type)
   eval(op1, type);
 }
 
-inline _bit::_bit(const _bit& op1, char type, int sh)
+inline _bit::_bit(const _bit& op1, char type, unsigned int sh)
   : len (_BITLEN_UNARY (op1.length(),op1.isUnsigned())),
     usgn(_BITUSGN_UNARY(op1.length(),op1.isUnsigned())),
     isDynamic(init())
@@ -776,28 +802,28 @@ inline _bit::_bit(const _bit& op1, char type, const _bit& op2)
 
 /* --- Operation results as a reference --- */
 
-inline _bit::_bit(chunk* ref, int length, bool usign,
+inline _bit::_bit(chunk* ref, len_t length, bool usign,
 		  const _bit& op1, char type)
   : vec(ref), len(length), usgn(usign), isDynamic(false) 
 {
   eval(op1, type);
 }
 
-inline _bit::_bit(chunk* ref, int length, bool usign,
+inline _bit::_bit(chunk* ref, len_t length, bool usign,
 		  const _bit& op1, char type, int sh) 
   : vec(ref), len(length), usgn(usign), isDynamic(false) 
 {
   eval(op1, type, sh);
 }
 
-inline _bit::_bit(chunk* ref, int length, bool usign,
+inline _bit::_bit(chunk* ref, len_t length, bool usign,
 		  const _bit& op1, const _bit& op2)
   : vec(ref), len(length), usgn(usign), isDynamic(false) 
 {
   eval(op1, op2);
 }
 
-inline _bit::_bit(chunk* ref, int length, bool usign,
+inline _bit::_bit(chunk* ref, len_t length, bool usign,
 		  const _bit& op1, char type, const _bit& op2)
   : vec(ref), len(length), usgn(usign), isDynamic(false) 
 {
@@ -809,21 +835,21 @@ inline _bit::_bit(chunk* ref, int length, bool usign,
 
 /* --- Slicing --- */
 
-inline _bitslice _bit::operator[] (int i) const {
+inline _bitslice _bit::operator[] (len_t i) const {
   return _bitslice(vec, len, usgn, i, i, true);
 }							    
 
-inline _bitslice _bit::operator()(int sl, int sr) const {
+inline _bitslice _bit::operator()(len_t sl, len_t sr) const {
   return _bitslice(vec, len, usgn, sl, sr, _BITUSGN_SLICE(len,usgn,sl,sr));
 }
 
 
-inline _bitslice _bit::operator()(int l, int r, int i) const {
+inline _bitslice _bit::operator()(bnd_t l, bnd_t r, bnd_t i) const {
   ASSERT(_BITLEN(l,r)==len);  
   return operator[](_BITMAP(l,r, i));
 }							    
 
-inline _bitslice _bit::operator()(int l, int r, int sl, int sr) const {
+inline _bitslice _bit::operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const {
   ASSERT(_BITLEN(l,r)==len);  
   return operator()(_BITMAP(l,r, sl), _BITMAP(l,r, sr));
 }
@@ -942,7 +968,7 @@ inline _bit& _bit::eval(const _bit& op1, char type) {
   return *this;
 }
 
-inline _bit& _bit::eval(const _bit& op1, char type, int sh) {
+inline _bit& _bit::eval(const _bit& op1, char type, unsigned int sh) {
   switch (type) {
     case '<': return shlOp(op1, sh);
     case '>': return shrOp(op1, sh);
@@ -1224,40 +1250,40 @@ inline _bit& _bit::fill(int chunk) {
 /* Class _bitslice inlined methods */
     
     
-inline  _bitslice _bitslice::operator[] (int i) {
-  return _bitslice(ref, _BITMAP(sl,sr,i), _BITMAP(sl,sr,i), true, lst);
+inline  _bitslice _bitslice::operator[] (len_t i) {
+  return _bitslice(ref, _BITSLICE(sl,sr,i), _BITSLICE(sl,sr,i), true, lst);
 }
 
-inline _bitslice _bitslice::operator()(int sl, int sr) {
-  return _bitslice(ref, _BITMAP(this->sl,this->sr, sl),
-		        _BITMAP(this->sl,this->sr, sr),
+inline _bitslice _bitslice::operator()(len_t sl, len_t sr) {
+  return _bitslice(ref, _BITSLICE(this->sl,this->sr, sl),
+		        _BITSLICE(this->sl,this->sr, sr),
 		        _BITUSGN_SLICE(ref.length(),isUnsigned(),sl,sr), 
 		        lst);
 }
 
-inline _bitslice _bitslice::operator()(int l, int r, int i) {
+inline _bitslice _bitslice::operator()(bnd_t l, bnd_t r, bnd_t i) {
   ASSERT(_BITLEN(l,r)==length());
   return operator[](_BITMAP(l,r, i));
 }
 
-inline _bitslice _bitslice::operator()(int l, int r, int sl, int sr) {
+inline _bitslice _bitslice::operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) {
   ASSERT(_BITLEN(l,r)==length()); 
   return operator()(_BITMAP(l,r, sl), _BITMAP(l,r, sr));
 }
 
-inline const _bitslice _bitslice::operator[] (int i) const {
+inline const _bitslice _bitslice::operator[] (len_t i) const {
   return const_cast<_bitslice*>(this)->operator[](i);  
 }
 
-inline const _bitslice _bitslice::operator()(int sl, int sr) const {
+inline const _bitslice _bitslice::operator()(len_t sl, len_t sr) const {
   return const_cast<_bitslice*>(this)->operator()(sl, sr);
 }
 
-inline const _bitslice _bitslice::operator()(int l, int r, int i) const {
+inline const _bitslice _bitslice::operator()(bnd_t l, bnd_t r, bnd_t i) const {
   return const_cast<_bitslice*>(this)->operator()(l, r, i);
 }
 
-inline const _bitslice _bitslice::operator()(int l, int r, int sl, int sr) const {
+inline const _bitslice _bitslice::operator()(bnd_t l, bnd_t r, bnd_t sl, bnd_t sr) const {
   return const_cast<_bitslice*>(this)->operator()(l, r, sl, sr);
 }
 
